@@ -28,6 +28,8 @@ from fabric.contrib.project import rsync_project
 from littlechef import lib
 from littlechef import solo
 from littlechef.settings import node_work_path, cookbook_paths
+from littlechef import LOGFILE, enable_logs as ENABLE_LOGS
+
 
 # Path to local patch
 basedir = os.path.abspath(os.path.dirname(__file__).replace('\\', '/'))
@@ -74,9 +76,12 @@ def sync_node(node):
         solo.configure(current_node)
         # Everything was configured alright, so save the node configuration
         filepath = save_config(node, _get_ipaddress(node))
-        _synchronize_node(filepath)
-        _remove_node_data_bag()
-        _configure_node()
+        try:
+            _synchronize_node(filepath)
+            _configure_node()
+        finally:
+            _remove_local_node_data_bag()
+            _node_cleanup()
 
 
 def _synchronize_node(configfile):
@@ -92,7 +97,7 @@ def _synchronize_node(configfile):
     remote_file = '/etc/chef/node.json'
     put(configfile, remote_file, use_sudo=True, mode=400)
     with hide('stdout'):
-        sudo('chown root:root {0}'.format(remote_file)),
+        sudo('chown root:root {0}'.format(remote_file))
     # Remove local temporary node file
     os.remove(configfile)
     # Synchronize kitchen
@@ -178,11 +183,10 @@ def _add_merged_attributes(node, all_recipes, all_roles):
                         value = {}
                     else:
                         value = r['attributes'][attr].get('default')
-                    # Attribute dictionaries are defined as a single 
+                    # Attribute dictionaries are defined as a single
                     # compound key. Split and build proper dict
                     build_dct(attributes, attr.split("/"), value)
         if not found:
-            _remove_node_data_bag()
             error = "Could not find recipe '{0}' while ".format(recipe)
             error += "building node data bag for '{0}'".format(node['name'])
             abort(error)
@@ -235,7 +239,8 @@ def _build_node_data_bag():
     current_node = None
     nodes = lib.get_nodes()
     node_data_bag_path = os.path.join('data_bags', 'node')
-    _remove_node_data_bag()
+    # In case there are leftovers
+    _remove_local_node_data_bag()
     os.makedirs(node_data_bag_path)
     all_recipes = lib.get_recipes()
     all_roles = lib.get_roles()
@@ -270,13 +275,28 @@ def _build_node_data_bag():
     return current_node
 
 
-def _remove_node_data_bag():
-    """Removes generated 'node' data_bag"""
+def _remove_local_node_data_bag():
+    """Removes generated 'node' data_bag locally"""
     node_data_bag_path = os.path.join('data_bags', 'node')
     if os.path.exists(node_data_bag_path):
         shutil.rmtree(node_data_bag_path)
 
 
+def _remove_remote_node_data_bag():
+    """Removes generated 'node' data_bag from the remote node"""
+    node_data_bag_path = os.path.join(node_work_path, 'data_bags', 'node')
+    if exists(node_data_bag_path):
+        sudo("rm -rf {0}".format(node_data_bag_path))
+
+
+def _node_cleanup():
+    if env.loglevel is not "debug":
+        with hide('running', 'stdout'):
+            _remove_remote_node_data_bag()
+            with settings(warn_only=True):
+                sudo("rm '/etc/chef/node.json'")
+
+
 def _add_search_patch():
     """ Adds chef_solo_search_lib cookbook, which provides a library to read and search
     data bags.
@@ -295,21 +315,28 @@ def _add_search_patch():
 def _configure_node():
     """Exectutes chef-solo to apply roles and recipes to a node"""
     print("\nCooking...")
+    # Backup last report
+    with settings(hide('stdout', 'warnings', 'running'), warn_only=True):
+        sudo("mv {0} {0}.1".format(LOGFILE))
+    # Build chef-solo command
+    cmd = 'chef-solo -l {0} -j /etc/chef/node.json'.format(env.loglevel)
+    if ENABLE_LOGS:
+        cmd += ' | tee {0}'.format(LOGFILE)
     with settings(hide('warnings', 'running'), warn_only=True):
-        output = sudo(
-            'chef-solo -l {0} -j /etc/chef/node.json'.format(env.loglevel))
-        if output.failed:
-            if 'chef-solo: command not found' in output:
-                print(
-                    colors.red(
-                        "\nFAILED: Chef Solo is not installed on this node"))
-                print(
-                    "Type 'fix nodes:{0} deploy_chef' to install it".format(
-                        env.host))
-                abort("")
-            else:
-                print(colors.red(
-                    "\nFAILED: A problem occurred while executing chef-solo"))
-                abort("")
+        output = sudo(cmd)
+    if output.failed or "FATAL: Stacktrace dumped" in output:
+        if 'chef-solo: command not found' in output:
+            print(
+                colors.red(
+                    "\nFAILED: Chef Solo is not installed on this node"))
+            print(
+                "Type 'fix nodes:{0} deploy_chef' to install it".format(
+                    env.host))
+            abort("")
         else:
-            print(colors.green("\nSUCCESS: Node correctly configured"))
+            print(colors.red(
+                "\nFAILED: chef-solo could not finish configuring the node\n"))
+            import sys
+            sys.exit(1)
+    else:
+        print(colors.green("\nSUCCESS: Node correctly configured"))
