@@ -34,14 +34,7 @@ import fabric
 fabric.state.output['running'] = False
 env.loglevel = "info"
 env.output_prefix = False
-__testing__ =  False
-
-
-@hosts('setup')
-def debug():
-    """Sets logging level to debug"""
-    print "Setting Chef Solo log level to 'debug'..."
-    env.loglevel = 'debug'
+__testing__ = False
 
 
 @hosts('setup')
@@ -75,8 +68,8 @@ def new_kitchen():
 
 @hosts('setup')
 def nodes_with_role(rolename):
-    """Sets a list of nodes that contain the given role in their run list
-    and calls node()
+    """Sets a list of nodes that have the given role
+    in their run list and calls node()
 
     """
     nodes_in_env = []
@@ -220,9 +213,8 @@ def get_ips():
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         resp, error = proc.communicate()
         if not error:
-            # Get lines from output and parse the first line to get the IP
-            lines = resp.split("\n")
-            ip = lines[0].split()[2].lstrip("(").rstrip(")")
+            # Split output into lines and parse the first line to get the IP
+            ip = lib.parse_ip(resp.split("\n")[0])
             if not ip:
                 print "Warning: could not get IP address from node {0}".format(
                     node['name'])
@@ -387,6 +379,7 @@ def _readconfig():
 # Only read config if fix is being used and we are not creating a new kitchen
 import littlechef
 env.chef_environment = littlechef.chef_environment
+env.loglevel = littlechef.loglevel
 
 if littlechef.__cooking__:
     # Called from command line
